package com.jsowell.web.controller.pile;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.jsowell.pile.dto.PileAuthCardDTO;
import com.jsowell.pile.vo.web.PileAuthCardVO;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.jsowell.common.annotation.Log;
import com.jsowell.common.core.controller.BaseController;
import com.jsowell.common.core.domain.AjaxResult;
import com.jsowell.common.enums.BusinessType;
import com.jsowell.pile.domain.PileAuthCard;
import com.jsowell.pile.service.IPileAuthCardService;
import com.jsowell.common.util.poi.ExcelUtil;
import com.jsowell.common.core.page.TableDataInfo;

/**
 * 充电站鉴权卡Controller
 *
 * @author jsowell
 * @date 2023-03-16
 */
@RestController
@RequestMapping("/pile/card")
public class PileAuthCardController extends BaseController {
    @Autowired
    private IPileAuthCardService pileAuthCardService;

    /**
     * 查询充电站鉴权卡列表
     */
    @PreAuthorize("@ss.hasPermi('pile:card:list')")
    @GetMapping("/list")
    public TableDataInfo list(PileAuthCardDTO dto) {
        startPage();
        // List<PileAuthCard> list = pileAuthCardService.selectPileAuthCardList(pileAuthCard);
        List<PileAuthCardVO> list = pileAuthCardService.getPileAuthCardInfoList(dto);
        return getDataTable(list);
    }

    /**
     * 导出充电站鉴权卡列表
     */
    @PreAuthorize("@ss.hasPermi('pile:card:export')")
    @Log(title = "充电站鉴权卡", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PileAuthCard pileAuthCard) {
        List<PileAuthCard> list = pileAuthCardService.selectPileAuthCardList(pileAuthCard);
        ExcelUtil<PileAuthCard> util = new ExcelUtil<PileAuthCard>(PileAuthCard.class);
        util.exportExcel(response, list, "充电站鉴权卡数据");
    }

    /**
     * 获取充电站鉴权卡详细信息
     */
    @PreAuthorize("@ss.hasPermi('pile:card:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(pileAuthCardService.getCardInfoById(id));
    }

    /**
     * 新增充电站鉴权卡
     */
    // @PreAuthorize("@ss.hasPermi('pile:card:add')")
    // @Log(title = "充电站鉴权卡", businessType = BusinessType.INSERT)
    // @PostMapping
    // public AjaxResult add(@RequestBody PileAuthCard pileAuthCard) {
    //     return toAjax(pileAuthCardService.insertPileAuthCard(pileAuthCard));
    // }

    /**
     * 新增充电站鉴权卡
     */
    @PreAuthorize("@ss.hasPermi('pile:card:add')")
    @Log(title = "充电站鉴权卡", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody PileAuthCardDTO dto) {
        return toAjax(pileAuthCardService.addAuthCard(dto));
    }

    /**
     * 修改充电站鉴权卡
     */
    @PreAuthorize("@ss.hasPermi('pile:card:edit')")
    @Log(title = "充电站鉴权卡", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody PileAuthCardDTO dto) {
        return toAjax(pileAuthCardService.updateCardInfo(dto));
    }

    /**
     * 删除充电站鉴权卡
     */
    @PreAuthorize("@ss.hasPermi('pile:card:remove')")
    @Log(title = "充电站鉴权卡", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(pileAuthCardService.deletePileAuthCardByIds(ids));
    }

    /**
     * 停用充电站鉴权卡
     */
    @PreAuthorize("@ss.hasPermi('pile:card:edit')")
    @Log(title = "充电站鉴权卡", businessType = BusinessType.UPDATE)
    @GetMapping("/deactivateCard/{id}")
    public AjaxResult deactivateCard(@PathVariable("id") Long id) {
        return toAjax(pileAuthCardService.deactivateCard(id));
    }
}
